﻿/// <reference path="/resources/JQuery/jquery-1.7-vsdoc.js" />

function getHashValue(key) {
    key = key.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regex = new RegExp("[\\#&]" + key + "=([^&#]*)");
    var qs = regex.exec(window.location.hash);
    if (qs == null) {
        return undefined;
    }
    else {
        return decodeURIComponent(qs[1]);
    }
};

function redirectWithHashValues(o) {
    // valid properties behind the hash sign 
    o.s = (o.s === undefined) ? getHashValue('s') : o.s;
    o.p = (parseInt(o.p) === NaN) ? getHashValue('p') : o.p;

    // iterate valid properties and mimic a query string with a # instead of a ?
    var url = window.location.protocol + '//' + window.location.host + window.location.pathname + '#';
    $.each(o, function (propName, propValue) {
        if (propValue !== undefined && propValue !== 0) {
            url += propName + '=' + encodeURIComponent(propValue) + '&';
        }
    });
    // remove trailing # if there are no key/value pairs, or & if there is at least one
    url = url.substring(0, url.length - 1);

    window.location = url;
};

function getIncludePrerelease() {
    return $('#prerelease-picker select').val() == 'prerelease';
};

function getVersionFilterQuery() {
    return getIncludePrerelease() ? 'IsAbsoluteLatestVersion' : 'IsLatestVersion';
};

function appendPackageToContainer(pkg, container, baseUrl) {
    var packageHtml = '<div class="package-container">'
        + '<div class="icon-container">';
    if (!pkg.IsProGetHosted) {
        packageHtml += '<img src="/Resources/Images/Icons/package-remote.30.png" />';
    } else if (pkg.IsProjectPackage) {
        packageHtml += '<img src="/Resources/Images/Icons/package-project.30.png" />';
    } else if (pkg.IsDeployPackage) {
        packageHtml += '<img src="/Resources/Images/Icons/package-deploy.30.png" />';
    }

    var tags = (pkg.Tags || '').split(' ');
    var anyTags = false;
    var tagsContainer = $('<div class="tags">Tags: </div>');
    for (var i = 0; i < tags.length; i++) {
        if (tags[i] != '') {
            anyTags = true;
            tagsContainer.append($('<span></span>').text(tags[i]));
        }
    }

    packageHtml += '</div>'
        + '<div class="description">'
            + '<h3><a href="' + baseUrl + '/' + pkg.Id + '/' + pkg.Version + '">' + pkg.Id + ' ' + pkg.Version + '</a></h3>'
            + '<p>' + pkg.Description + '</p>';

    if (anyTags) {
        packageHtml += tagsContainer[0].outerHTML;
    }

    packageHtml += '</div>'
      + '</div>';

    var newPackage = $(packageHtml);

    if (pkg.IconUrl) {
        newPackage.find('.icon-container').css('background-image', 'url(\'' + pkg.IconUrl + '\')');
    }

    container.append(newPackage);
};

proget_browse = function (o) {
    var pos = 0;
    var container = $('.packages-container');
    var totalCount = null;
    var countUrl;

    var ajaxReloadTotalCount = function (async) {
        if (getHashValue('s')) {
            countUrl = o.apiUrl + "/Search()/$count?$filter=" + getVersionFilterQuery() + "&searchTerm='" + getHashValue('s') + "'&targetFramework=''&includePrerelease=" + getIncludePrerelease();
        } else {
            countUrl = o.apiUrl + '/Packages/$count?$filter=' + getVersionFilterQuery();
        }
        $.ajax({
            url: countUrl,
            cache: false,
            async: async,
            success: function (data) {
                totalCount = data;
                var orig = $('.packages-heading').text();
                if (parseInt(totalCount) > 0) {
                    $('.packages-heading').text(orig + ' of ' + totalCount);
                }
            }
        });
    };
    ajaxReloadTotalCount(true);

    var updateTitle = function (reloadTotalCount) {
        if (reloadTotalCount) {
            ajaxReloadTotalCount(false);
        }

        var titleText = 'Displaying packages ' + (pos + 1) + '-' + Math.min(totalCount, pos + 20);
        if (parseInt(totalCount) > 0) {
            titleText += ' of ' + totalCount;
        } else {
            titleText = 'No packages found.';
            $('.package-nav-links').hide();
        }
        $('.packages-heading').text(titleText);
    };

    var getPackages = function () {
        pos = parseInt(getHashValue('p') || 0);
        var url;
        if (getHashValue('s')) {
            url = o.apiUrl + '/Search()/?$format=json&$filter=' + getVersionFilterQuery() + '&$orderby=DownloadCount%20desc&$top=20&$skip=' + pos + '&searchTerm=\'' + encodeURIComponent(getHashValue('s')) + '\'&targetFramework=\'\'&includePrerelease=' + getIncludePrerelease();
        } else {
            url = o.apiUrl + '/Packages/?$format=json&$filter=' + getVersionFilterQuery() + '&$orderby=DownloadCount%20desc&$top=20&$skip=' + pos;
        }

        $('.loading').show();
        container.fadeOut('fast');
        container.children().remove();
        $('.package-nav-links').hide();
        $('.package-nav-links > a').addClass('inactive');
        $.ajax({
            url: url,
            cache: false,
            success: function (data, textStatus, jqXHR) {

                if (jqXHR.getResponseHeader('x-pgconnectorerror')) {
                    $(".info-box.warning").show();
                    $(".info-box.warning p").text('There was an error with one or more of your connectors: ' + jqXHR.getResponseHeader('x-pgconnectorerror'));
                }

                container.hide();
                container.children().remove();
                var packages = data.d.results;
                for (var i = 0; i < packages.length; i++) {
                    appendPackageToContainer(packages[i], container, o.baseUrl);
                }

                if (pos > 0) {
                    $('.package-nav-links a:first-child').removeClass('inactive');
                }

                if (packages.length >= 20) {
                    $('.package-nav-links a:last-child').removeClass('inactive');
                }

                container.fadeIn('fast');

                $('.loading').hide();

                $('.package-nav-links').show();

                updateTitle(true);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                $(".info-box.error").show();
                $(".info-box.error p").text('An unexpected error has occurred: ' + jqXHR.responseText);
                $('.loading').hide();
                console.log(jqXHR);
                console.log(textStatus);
                console.log(errorThrown);
            }
        });
    };

    $('.package-nav-links a:first-child').click(function () {
        if (!$(this).hasClass('inactive')) {
            pos -= 20;
            if (pos < 0) {
                pos = 0;
            }
            redirectWithHashValues({ p: pos });
        }
        return false;
    });

    $('.package-nav-links a:last-child').click(function () {
        if (!$(this).hasClass('inactive')) {
            pos += 20;
            redirectWithHashValues({ p: pos });
        }
        return false;
    });

    $('form').submit(function () {

        var searchTerm = $('.package-search input').val();
        redirectWithHashValues({ s: searchTerm });

        return false;
    });

    $('.package-search img').click(function () {
        $('form').submit();
    });

    $(window).bind('hashchange', function () {
        getPackages();
    });

    $('#prerelease-picker select').change(function () {
        getPackages();
    });

    getPackages();
};